#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <assert.h>

struct etudiant {
	char* prenom;
	unsigned int age;
};
typedef struct etudiant etu_t;

typedef struct equipe{
	unsigned int nb_membres;
	etu_t** membres; // tableaux de pointeurs vers les membres de l'equipe
	unsigned int indice_cpt; // indice du capitaine dans le tableau des membres
} equipe_t;

/* Affiche les informations de e, sans retour ligne à la fin */
void print_etu(etu_t* e){
	printf("%s (%d ans)", e->prenom, e->age);
}

/* Affiche les informations des membres de e */
void print_equipe(equipe_t* e){
	printf("************\n");
	for (unsigned int i = 0; i < e->nb_membres; ++i)
	{
		if (i == e->indice_cpt){
			printf("Capitaine: ");
		} else {
			printf("           ");
		}
		print_etu(e->membres[i]);
		printf("\n");
	}
	printf("************\n");
}

/* Libère toute la mémoire allouée à un etu_t */
void free_etu(etu_t* e){
	free(e);
}


/* Libère toute la mémoire allouée à une equipe_t, y
   compris pour ses membres */
void free_equipe(equipe_t* e){
	for (unsigned int i = 0; i < e->nb_membres; ++i)
	{
		free(e->membres[i]);
	}
	free(e->membres);
	free(e);
}

/* Rajoute les membres de l'équipe src à l'équipe dst.
   Après cette fonction, src est libérée et donc invalide. */
void agrandir_equipe(equipe_t* dst, equipe_t* src){
	int nouv_taille = dst->nb_membres + src->nb_membres;
	dst->membres = realloc(dst->membres, nouv_taille*sizeof(etu_t*));
	for (unsigned int i = 0; i < src->nb_membres; ++i)
	{
		dst->membres[dst->nb_membres + i] = src->membres[i];
	}

	if (dst->nb_membres < src->nb_membres){
		dst->indice_cpt = dst->nb_membres + src->indice_cpt;
	}

	dst->nb_membres = nouv_taille;

	free(src->membres);
	free(src);
}

int main(){

	etu_t* cam = malloc(sizeof(etu_t));
	cam->prenom = "Camille";
	cam->age = 23;

	etu_t* lei = malloc(sizeof(etu_t));
	lei->prenom = "Leila";
	lei->age = 20;

	etu_t* thi = malloc(sizeof(etu_t));
	thi->prenom = "Thibault";
	thi->age = 22;

	equipe_t* e = malloc(sizeof(equipe_t));
	e->nb_membres = 3;
	e->indice_cpt = 1;
	e->membres = malloc(e->nb_membres*sizeof(etu_t*));

	e->membres[0] = cam;
	e->membres[1] = lei;
	e->membres[2] = thi;

	equipe_t* e2 = malloc(sizeof(equipe_t));
	e2->nb_membres = 3;
	e2->membres = malloc(3*sizeof(etu_t*));
	
	e2->membres[0] = malloc(sizeof(etu_t));
	e2->membres[0]->prenom = "Caroline";
	e2->membres[0]->age = 24;

	e2->membres[1] = malloc(sizeof(etu_t));
	e2->membres[1]->prenom = "Olivier";
	e2->membres[1]->age = 23;

	e2->membres[2] = malloc(sizeof(etu_t));
	e2->membres[2]->prenom = "Guillaume";
	e2->membres[2]->age = 25;

	e2->indice_cpt = 2;

	agrandir_equipe(e, e2);

	print_equipe(e);
	free_equipe(e);
	return 0;
}


