#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <assert.h>


struct maillon{
	struct maillon* suivant;
	int val;
};
typedef struct maillon maillon_t;

struct liste {
	unsigned int taille;
	maillon_t* tete; // premier maillon
};

typedef struct liste liste_t;

/* Renvoie une liste contenant 0, 1, ..., n-1 (liste vide si n <= 0) */
liste_t* range(int n){
	liste_t* res = malloc(sizeof(liste_t));
	res->taille = n;
	if (n == 0){
		res->tete = NULL;
		return res;
	}

	res->tete = malloc(sizeof(maillon_t));
	maillon_t* courant = res->tete;
	courant->val = 0;
	for (int i = 1; i < n; ++i)
	{
		courant->suivant = malloc(sizeof(maillon_t));
		courant = courant->suivant;
		courant->val = i;
	}
	courant->suivant = NULL;
	return res;
}

/* Affiche les valeurs de l dans l'ordre */
void print_liste(liste_t* l){
	assert(l != NULL);
	// parcourir la liste depuis la tête jusqu'au maillon nul
	for (maillon_t* m = l->tete; m != NULL; m = m->suivant){
		printf("%d ", m->val);
	}
	printf("\n");
}


/* Libère la mémoire allouée à l */
void free_liste(liste_t* l){
	assert(l != NULL);
	maillon_t* m = l->tete;
	while (m != NULL){
		maillon_t* prec = m;		
		m = m->suivant;
		free(prec);
	}
	free(l);
}

/* Détermine si x est dans l*/
bool recherche(int x, liste_t* l){
	assert(l != NULL);
	for (maillon_t* m = l->tete; m != NULL; m = m->suivant){
		if (m->val == x){
			return true;
		}
	}
	return false;
}

/* Ajoute x à la fin de l */
void ajout_fin(int x, liste_t* l){
	assert(l != NULL);
	// si la liste est vide: creer un maillon de tete
	if (l->tete == NULL){
		l->tete = malloc(sizeof(maillon_t));
		l->tete->suivant = NULL;
		l->tete->val = x;
		return;
	}

	// sinon: trouver le dernier maillon et le faire suivre
	// par un nouveau maillon contenant x
	maillon_t* m = l->tete;
	while (m->suivant != NULL){
		m = m->suivant;
	}
	// en sortie: m n'est pas NULL mais m->suivant est NULL
	m->suivant = malloc(sizeof(maillon_t));
	m->suivant->suivant = NULL;
	m->suivant->val = x;
}

/* Ajoute x au début de l */
void ajout_debut(int x, liste_t* l){
	assert(l != NULL);
	// sauvegarder l'ancienne tete, et créer un nouveau
	// maillon dont le suivant est l'ancienne tete
	maillon_t* anc_tete = l->tete;
	l->tete = malloc(sizeof(maillon_t));
	l->tete->suivant = anc_tete;
	l->tete->val = x;
}

int main(int argc, char const *argv[])
{
	liste_t* l = range(8);
	print_liste(l);
	printf("6 est-il dans la liste: %d\n", recherche(6, l));
	printf("102 est-il dans la liste: %d\n", recherche(102, l));

	ajout_fin(87, l);
	ajout_debut(102, l);

	print_liste(l);
	printf("6 est-il dans la liste: %d\n", recherche(6, l));
	printf("102 est-il dans la liste: %d\n", recherche(102, l));

	free_liste(l);
	return 0;
}